/**************************************************************************
*  Compilation:  javac PrintSeams.java
*  Execution:    java PrintSeams input.png
*  Dependencies: SeamCarver.java 
*
*  Read image from file specified as command-line argument. Print square
*  of energies of pixels, a vertical seam, and a horizontal seam.
*
*  % java PrintSeams 6x5.png
*  6x5.png (6-by-5 image)
*  
*  The table gives the dual-gradient energies of each pixel.
*  The asterisks denote a minimum energy vertical or horizontal seam.
*  
*  Vertical seam: { 3 4 3 2 2 }
*   240.18   225.59   302.27   159.43*  181.81   192.99  
*   124.18   237.35   151.02   234.09   107.89*  159.67  
*   111.10   138.69   228.10   133.07*  211.51   143.75  
*   130.67   153.88   174.01*  284.01   194.50   213.53  
*   179.82   175.49    70.06*  270.80   201.53   191.20  
*  Total energy = 644.467988
*  
*  
*  Horizontal seam: { 2 2 1 2 1 2 }
*   240.18   225.59   302.27   159.43   181.81   192.99  
*   124.18   237.35   151.02*  234.09   107.89*  159.67  
*   111.10*  138.69*  228.10   133.07*  211.51   143.75* 
*   130.67   153.88   174.01   284.01   194.50   213.53  
*   179.82   175.49    70.06   270.80   201.53   191.20  
*  Total energy = 785.531820
*
**************************************************************************/

public class PrintSeams {

  public static final boolean HORIZONTAL   = true;
  public static final boolean VERTICAL     = false;

  public static void printSeam(SeamCarver carver, int[] seam, 
                               boolean direction) {
    double totalSeamEnergy = 0.0;

    for (int row = 0; row < carver.height(); row++) {
        for (int col = 0; col < carver.width(); col++) {
            double energy = carver.energy(col, row);
            String marker = " ";
            if ((direction == HORIZONTAL && row == seam[col]) ||
                (direction == VERTICAL   && col == seam[row])) {
                marker = "*";
                totalSeamEnergy += energy;
            }
            System.out.printf("%7.2f%s ", energy, marker);
        }
        System.out.println();
    }                
    // System.out.println();
    System.out.printf("Total energy = %f\n", totalSeamEnergy);
    System.out.println();
    System.out.println();
  }

  public static void main(String[] args) {
    Picture picture = new Picture(args[0]);
    System.out.printf("%s (%d-by-%d image)\n", args[0], 
                       picture.width(), picture.height());
    System.out.println();
    System.out.println("The table gives each pixel dual-gradient energy.");
    System.out.println("The asterisks denote a minimum energy " + 
                      "vertical seam or horizontal one.");
    System.out.println();

    SeamCarver carver = new SeamCarver(picture);
    
    int[] verticalSeam = carver.findVerticalSeam();
    
    if (verticalSeam != null) {
      System.out.printf("Vertical seam: { ");

        for (int x : verticalSeam)
            System.out.print(x + " ");
        System.out.println("}");
      
        printSeam(carver, verticalSeam, VERTICAL);
    } else {
        System.out.println("No seam found");  
    }

    /*
    System.out.printf("Horizontal seam: { ");
    int[] horizontalSeam = carver.findHorizontalSeam();
    for (int y : horizontalSeam)
        System.out.print(y + " ");
    System.out.println("}");
    printSeam(carver, horizontalSeam, HORIZONTAL);
*/
  }

}
